﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Statki.Model
{
    public enum GamePhase
    {
        PlanningPhase,
        ShootingPhase,
        ResumePhase
    }

    public class Gameplay
    {
        public GamePhase GamePhase { get; private set; }
        public int BoardDimension => 10;
        readonly public int[] ShipSizes = new[] { 4, 3, 3, 2, 2, 2, 1, 1, 1, 1 };
        public int ShipCurrentlyAdded { get; private set; } // Tu przechowujemy statek, który aktualnie gracz powinien dodać
        private int actualShipIndex; // aktualny index statku w tablicy ShipSizes[]
        public bool AllShipsAreAdded { get; private set; } // To pozwoli nam określić kiedy wszystkie statki są dodane i można zaczynać grę
        public Field[,] FieldsPlayer1 { get; private set; }
        public Field[,] FieldsPlayer2 { get; private set; }
        public PlayerEnum CurrentPlayer { get; private set; }
        public List<Ship> ShipCollectionPlayer1 { get; private set; }
        public List<Ship> ShipCollectionPlayer2 { get; private set; }
        AddRandomShip addRandomShip;
        // Metoda która po poprawnym dodaniu statku pozwoli określic który ma byc dodany następny, bądź czy był już to ostatni statek i możemy zacząć grę
        public void NextShipToAdd()
        {
            if (actualShipIndex<ShipSizes.Length - 1)
            {
               actualShipIndex++;
               ShipCurrentlyAdded = ShipSizes[actualShipIndex];
            }
            else
            {
                AllShipsAreAdded = true;
                
            }
        }

        public void LaunchAnotherPhase()
        {
            switch (GamePhase)
            {
                case GamePhase.PlanningPhase:
                    GamePhase = GamePhase.ShootingPhase;
                    break;
                case GamePhase.ShootingPhase:
                    GamePhase = GamePhase.ResumePhase;
                    break;
                case GamePhase.ResumePhase:
                    break;
                default:
                    break;
            }
        }
        //ToDo: Czy to powinna przechowywać ta klasa czy lepiej by gracz miał tu "władze"?
        /// <summary>
        /// Metoda dodaje nowo utworzony statek do kolekcji wybranego gracza
        /// </summary>
        /// <param name="ship">Nowy statek</param>
        /// <param name="playerEnum">Gracz który dodaje swój statek</param>
        public void AddShipToCollection(Ship ship, PlayerEnum playerEnum)
        {
            switch (playerEnum)
            {
                case PlayerEnum.PlayerOne:
                    ShipCollectionPlayer1.Add(ship);
                    break;
                case PlayerEnum.PlayerTwo:
                    ShipCollectionPlayer2.Add(ship);
                    break;
                default: break;
            }
        }

        public Gameplay()
        {
            CurrentPlayer = PlayerEnum.PlayerOne;

            AllShipsAreAdded = false;
            actualShipIndex = 0;
            ShipCurrentlyAdded = ShipSizes[actualShipIndex];

            ShipCollectionPlayer1 = new List<Ship>();
            ShipCollectionPlayer2 = new List<Ship>();

            FieldsPlayer1 = new Field[BoardDimension + 1, BoardDimension + 1];
            FieldsPlayer2 = new Field[BoardDimension + 1, BoardDimension + 1];

            for (int i = 0; i <= BoardDimension; i++)
                for (int j = 0; j <= BoardDimension; j++)
                {
                    FieldsPlayer1[i, j] = new Field(new Coordinate(i, j));
                    FieldsPlayer2[i, j] = new Field(new Coordinate(i, j));
                }
            addRandomShip = new AddRandomShip(FieldsPlayer2);
            ShipsForAI();
            
        }
        private void ShipsForAI()
        {
            List<Field> ActualShipForAI;
            
            for(int i = 0; i < ShipSizes.Length; i++)
            {
                ActualShipForAI = addRandomShip.RandCoordinates(ShipSizes[i]);
                Ship ship = new Ship(ActualShipForAI);
                ShipCollectionPlayer2.Add(ship);
                ActualShipForAI.Clear();
            }

        }
        //ToDo: Kto wykonuje ruch?
        public void UpdateField(Field field, PlayerEnum playerEnum)
        {
            if (CurrentPlayer == playerEnum) throw new ArgumentException("Nie ten gracz ", nameof(playerEnum));
            field.ShotField();
            UpdateTurn(playerEnum);
        }

        public void UpdateTurn(PlayerEnum playerEnum)
        {
            CurrentPlayer = playerEnum == PlayerEnum.PlayerOne ? PlayerEnum.PlayerTwo : PlayerEnum.PlayerOne;
        }

        //Notatki
        //yyy o tym wartunku zakończenia gry yyy jest zmienna która ma ilość statków
        /*
         jest zmienna, która przechowuje ilość statków i gdy ona zejdzie do zera to będzie
         wiadomo statki którego gracza zostały zatopione
        */
        /*
         * czy dany statek może być w danym miejscu - nowa klasa
         * ustawienie statków
         * strzelanie AI
         * czy jest już koniec gry
            Podział prac
                Jagoda: Shooting Phase
                Bartek: Planning Phase
                Mateusz: "Scalenie"
        */

    }
}
